@@ -35,13 +35,14 @@ import tempfile
 import logging
 import csv
 import AcraNetwork.MPEGTS as ampeg
+import base64
 
 THIS_DIR = os.path.dirname(os.path.abspath(__file__))
 TMP_DIR = tempfile.gettempdir()
 
 
 logging.basicConfig(level=logging.INFO)
-logging.info(f"Temp folder={TMP_DIR}")
+# logging.info(f"Temp folder={TMP_DIR}")
 
 
 def getEthernetPacket(data: bytes = b""):
@@ -264,7 +265,7 @@ class CH10UDPTest(unittest.TestCase):
             c.sequence = 0x15
             c.payload = get_ch10(16).pack()
             self.assertTrue(c.pack())
-            print(repr(c))
+            # print(repr(c))
             c2 = ch10udp.Chapter10UDP()
             c2.unpack(c.pack())
             rec.payload = getEthernetPacket(c.pack())
@@ -399,6 +400,7 @@ class Ch10UARTTest(unittest.TestCase):
         self.assertEqual(uart, uart2)
         self.assertEqual(repr(uart2), uart_pkt)
 
+    @unittest.skip("missing pcap")
     def test_uart_le_unpack(self):
         p = pcap.Pcap(os.path.join(THIS_DIR, "ch10_uart_le.pcap"))
         mypcaprecord = p[0]
@@ -598,13 +600,13 @@ class PCMData(unittest.TestCase):
         self.assertEqual(pcmdf, pcmdf2)
         pcmdf3 = ch10pcm.PCMDataPacket(syncword=None)
         self.assertTrue(pcmdf3.unpack(packed_data))
-        print(pcmdf3)
+        # print(pcmdf3)
 
     def test_pcm_throughput(self):
         pcmdf = ch10pcm.PCMDataPacket()
         pcmdf.channel_specific_word = 0x100000  #
         mf = ch10pcm.PCMMinorFrame(throughput=True)
-        mf.minor_frame_data = struct.pack("<HH", 0xFE6B, 0x2840)
+        mf.minor_frame_data = struct.pack("<HHI", 0xFE6B, 0x2840, 0)
         pcmdf.minor_frames.append(mf)
         packed = pcmdf.pack()
         pcmdf2 = ch10pcm.PCMDataPacket()
@@ -612,8 +614,9 @@ class PCMData(unittest.TestCase):
         self.assertEqual(pcmdf, pcmdf2)
         self.assertEqual(
             repr(pcmdf2),
-            "PCM Data Packet Format 1. Channel Specific Word =0X100000\nMinor Frame Throughput mode Time=None Payload_len=4\n",
+            "PCM Data Packet Format 1. Channel Specific Word =0X100000\nMinor Frame Throughput mode Time=None Payload_len=8\n",
         )
+        print(base64.b64encode(packed))
 
     def test_pcm_endianness(self):
         mf = ch10pcm.PCMMinorFrame(throughput=True)
@@ -638,7 +641,7 @@ class MnACQData(unittest.TestCase):
             if wrapper.type == ch10udp.Chapter10UDP.TYPE_SEG and wrapper.segmentoffset == 0 and ch10_payload != b"":
                 # segmented packet with existing payload
                 self.assertTrue(pkt.unpack(ch10_payload))
-                print(repr(pkt))
+                # print(repr(pkt))
                 ch10_payload = b""
                 self.assertEqual(pkt.datatype, DATA_TYPE_PCM_DATA_FMT1)
                 pcm = ch10pcm.PCMDataPacket()
@@ -662,7 +665,7 @@ class MnACQData(unittest.TestCase):
                 ch10_payload += wrapper.payload
             else:
                 pkt.unpack(wrapper.payload)
-                print(repr(pkt))
+                # print(repr(pkt))
 
         cf.close()
         p.close()
@@ -675,10 +678,10 @@ class CH10SampleFile(unittest.TestCase):
         total_len = 0
         with fileparser as chf:
             for pkt in chf:
-                print(repr(pkt))
+                # print(repr(pkt))
                 total_len += pkt.packetlen
                 # self.assertTrue(False)
-        print(f"{total_len:,d}")
+        # print(f"{total_len:,d}")
 
     @unittest.skip("Not a test")
     def test_read_ch10(self):
@@ -689,7 +692,7 @@ class CH10SampleFile(unittest.TestCase):
                 pkt_repr = repr(pkt)
                 total_len += pkt.packetlen
                 # self.assertTrue(False)
-        print(f"{total_len:,d}")
+        # print(f"{total_len:,d}")
 
     @unittest.skip("Not a test")
     def test_extract_tmats(self):
@@ -755,7 +758,7 @@ class Video(unittest.TestCase):
             mpgsts.append(mpegp)
         vid.mpegts = mpgsts
         buf = vid.pack()
-        print(repr(buf))
+        # print(repr(buf))
         vid2 = ch10video.VideoFormat2()
         vid2.unpack(buf)
         self.assertEqual(vid, vid2)
